@@ -1,11 +1,11 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """Test spectrum.py module and related functionalities."""
-from __future__ import absolute_import, division, print_function
 
 # STDLIB
 import os
 import shutil
 import tempfile
+import warnings
 
 # THIRD-PARTY
 import numpy as np
@@ -21,10 +21,12 @@ from astropy.modeling.models import (
     PowerLaw1D, RedshiftScaleFactor)
 from astropy.utils import minversion
 from astropy.utils.data import get_pkg_data_filename
+from astropy.utils.exceptions import AstropyUserWarning
 
 # LOCAL
 from .test_units import _area, _wave, _flux_jy, _flux_photlam, _flux_vegamag
 from .. import exceptions, units
+from ..compat import ASTROPY_LT_4_0
 from ..models import (
     BlackBodyNorm1D, Box1D, ConstFlux1D, Empirical1D, Gaussian1D,
     GaussianAbsorption1D, GaussianFlux1D, Lorentz1D, MexicanHat1D,
@@ -167,8 +169,10 @@ class TestEmpiricalSourceFromFile(object):
 
     def test_neg_flux(self):
         w = [1000, 5000, 9000]
-        sp = SourceSpectrum(
-            Empirical1D, points=w, lookup_table=[100, -45, 5e-17])
+        with pytest.warns(AstropyUserWarning,
+                          match=r'contained negative flux or throughput'):
+            sp = SourceSpectrum(
+                Empirical1D, points=w, lookup_table=[100, -45, 5e-17])
         np.testing.assert_array_equal(sp(w).value, [100, 0, 5e-17])
         assert 'NegativeFlux' in sp.warnings
 
@@ -646,11 +650,15 @@ class TestNormalize(object):
     def _compare_countrate(self, rn_sp, ans_countrate):
         # Observation is needed to compare with expected count rate
         # although it is tested in test_observation.py
-        obs = Observation(rn_sp, self.acs, force='extrap')
+        with warnings.catch_warnings():
+            warnings.filterwarnings(
+                'ignore', message=r'.*Source spectrum will be evaluated '
+                r'outside pre-defined waveset.*', category=AstropyUserWarning)
+            obs = Observation(rn_sp, self.acs, force='extrap')
         ct_rate = obs.countrate(_area)
 
-        # 0.1% agreement with IRAF SYNPHOT COUNTRATE
-        np.testing.assert_allclose(ct_rate.value, ans_countrate, rtol=1e-3)
+        # 0.2% agreement with IRAF SYNPHOT COUNTRATE
+        np.testing.assert_allclose(ct_rate.value, ans_countrate, rtol=2e-3)
 
     @pytest.mark.parametrize(
         ('sp_type', 'rn_val', 'ans_countrate'),
@@ -706,7 +714,9 @@ class TestNormalize(object):
         """Test force=True for 'partial_notmost' overlap."""
         sp = SourceSpectrum(Empirical1D, points=[5000, 6000],
                             lookup_table=[1, 1])
-        rn_sp = sp.normalize(1e-23 * u.Jy, band=self.acs, force=True)
+        with pytest.warns(AstropyUserWarning,
+                          match=r'Spectrum is not defined everywhere'):
+            rn_sp = sp.normalize(1e-23 * u.Jy, band=self.acs, force=True)
         assert 'PartialRenorm' in rn_sp.warnings
         assert 'PartialRenorm' not in sp.warnings
 
@@ -717,7 +727,9 @@ class TestNormalize(object):
     def test_renorm_partial_most(self):
         """Test 'partial_most' overlap."""
         bp = SpectralElement(Box1D, amplitude=1, x_0=5600, width=870)
-        rn_sp = self.em.normalize(1e-23 * u.Jy, band=bp)
+        with pytest.warns(AstropyUserWarning,
+                          match=r'Spectrum is not defined everywhere'):
+            rn_sp = self.em.normalize(1e-23 * u.Jy, band=bp)
         assert 'PartialRenorm' in rn_sp.warnings
         assert 'PartialRenorm' not in self.em.warnings
         assert '99%' in rn_sp.warnings['PartialRenorm']
@@ -757,11 +769,18 @@ class TestWaveset(object):
     def test_box1d(self):
         bp = SpectralElement(Box1D, x_0=2000, width=1)
         w = bp.waveset.value
+        w_true = bp.model.sampleset()
 
-        np.testing.assert_array_equal(w, bp.model.sampleset())
+        np.testing.assert_array_equal(w, w_true)
         np.testing.assert_allclose(
             w[([0, 1, -2, -1], )], bp.model.sampleset(minimal=True))
 
+        # Make sure scale does not change waveset
+        bp2 = bp * 2
+        bp3 = 0.5 * bp
+        np.testing.assert_array_equal(bp2.waveset.value, w_true)
+        np.testing.assert_array_equal(bp3.waveset.value, w_true)
+
     def test_composite_none(self):
         bp1 = SpectralElement(Box1D, amplitude=1, x_0=5000, width=10)
         bp2 = SpectralElement(Const1D, amplitude=2)
@@ -794,6 +813,31 @@ class TestWaveset(object):
         sp = SourceSpectrum(Const1D, amplitude=1, z=1.3)
         assert sp.waveset is None
 
+    def test_complicated_tree(self):
+        """Throw everything in and insert redshift and scale in the middle."""
+
+        # On one side, we have a composite bandpass.
+        bp1 = SpectralElement(Const1D, amplitude=1.01)
+        bp2 = SpectralElement(
+            Empirical1D, points=[4999, 5000.001, 5030], lookup_table=[0, 1, 0])
+        bp = bp1 * (0.8 * bp2)  # [4999, 5000.001, 5030]
+
+        # On the other side, we have composite spectrum with
+        # scale and redshift.
+        sp1 = SourceSpectrum(
+            Empirical1D, points=[5001, 5011, 5020], lookup_table=[0, 1, 0])
+        sp2 = SourceSpectrum(
+            Empirical1D, points=[5000, 5010, 5020], lookup_table=[0, 1, 0])
+        sp3 = sp2 + (sp1 * 0.5)  # [5000, 5001, 5010, 5011, 5020]
+        sp3.z = 0.01  # [5050, 5051.01, 5060.1, 5061.11, 5070.2]
+        sp = sp1 + sp3  # [5001, 5011, 5020, 5050, 5051.01, 5060.1, 5061.11, 5070.2]  # noqa
+
+        sp_final = sp * bp
+        np.testing.assert_array_equal(
+            sp_final.waveset.value,
+            [4999, 5000.001, 5001, 5011, 5020, 5030, 5050, 5051.01,
+             5060.1, 5061.11, 5070.2])
+
     def test_exceptions(self):
         with pytest.raises(exceptions.SynphotError):
             get_waveset('foo')
@@ -826,7 +870,10 @@ class TestRedShift(object):
         assert self.sp_z0.z_type == self.sp.z_type == 'wavelength_only'
 
         assert isinstance(self.sp_z0.model, Gaussian1D)
-        assert isinstance(self.sp.model, modeling.core._CompoundModel)
+        if ASTROPY_LT_4_0:
+            assert isinstance(self.sp.model, modeling.core._CompoundModel)
+        else:
+            assert isinstance(self.sp.model, modeling.core.CompoundModel)
 
     def test_composite_redshift(self):
         sp2 = self.sp_z0 + self.sp  # centers: 5000, 11500
